/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#ifndef LW_GUI_COMMON_H
#define LW_GUI_COMMON_H

#include <stdlib.h>
#include <gtk/gtk.h>


inline void launch_app (GtkWidget *, gpointer);

inline void core_vanished (GDBusConnection *, const gchar *, GtkWidget *);
inline void core_appeared (GDBusConnection *con, const gchar *name, const gchar *owner, GtkWidget *w);

inline void  active_plugin_get (GtkWidget *, GSettings *);
inline void _active_plugin_get (GSettings *, gchar *, GSList *);

inline void   active_plugin_set (GtkWidget *, GSettings *);
inline void  _active_plugin_set (GtkWidget *, GSettings *);


inline void
launch_app (GtkWidget *w, gpointer app)
{
	(void) w;
#if GLIB_CHECK_VERSION(2,32,0)
    g_thread_new (NULL, (GThreadFunc)system, app);
#else
    g_thread_create (   (GThreadFunc)system, app, FALSE, NULL);
#endif
}

/* Livewallpaper button functions */
inline void
core_vanished (GDBusConnection *con, const gchar *name, GtkWidget *w)
{
	(void) con; (void) name;
	g_object_set_data (G_OBJECT(w), "core_started", GINT_TO_POINTER(FALSE));
	
#ifdef LWP_CONFIG
	gtk_switch_set_active (GTK_SWITCH (w), FALSE);
#else
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), FALSE);
#endif
}
inline void
core_appeared (GDBusConnection *con, const gchar *name, const gchar *owner, GtkWidget *w)
{
	(void) con; (void) name; (void) owner;
	g_object_set_data (G_OBJECT(w), "core_started", GINT_TO_POINTER(TRUE));
}

inline void
livewallpaper_button_activated (GtkWidget *w , GSettings *s)
{
	if (! GPOINTER_TO_INT(g_object_get_data (G_OBJECT (w), "core_started")) &&
		  g_settings_get_boolean (s, "active"))
			launch_app (NULL, "livewallpaper");
}


inline void
active_plugin_get (GtkWidget *w, GSettings *s)
{
	/* Check if the widget's embeded string is equals to
	 * the name of the active-plugin property, and mark
	 * the widget as active if it is so. */
	if (! strcmp (g_settings_get_string (s, "active-plugin"), 
				  g_object_get_data (G_OBJECT(w), "plugin")))
#ifdef LWP_CONFIG
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
#else
			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
#endif
}
inline void
_active_plugin_get (GSettings *s, gchar *key, GSList *l)
{
	/* Call active_plugin_get foreach element of the GSList */
	g_slist_foreach (l, (GFunc)active_plugin_get, s);
	(void) key;
}

inline void
active_plugin_set (GtkWidget *w, GSettings *s)
{
	/* Check if the widget is active, and if it is so,
	 * set the active-plugin property to its new value */
#ifdef LWP_CONFIG
	if ( gtk_toggle_button_get_active   (GTK_TOGGLE_BUTTON (w)))
#else
	if ( gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (w)))
#endif
		g_settings_set_string (s, "active-plugin",
							   g_object_get_data (G_OBJECT(w), "plugin"));
}
inline void
_active_plugin_set (GtkWidget *w, GSettings *s)
{
	/* Get the GSList associated to the widget activated and
	 * call active_plugin_set foreach element of this GSList */
#ifdef LWP_CONFIG
	GSList *l = gtk_radio_button_get_group (GTK_RADIO_BUTTON (w));
#else
	GSList *l = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (w));
#endif
	g_slist_foreach (l, (GFunc)active_plugin_set, s);
}

#endif /* LW_GUI_COMMON_H */
