/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "add_key.h"

static gboolean s_to_rgba  (GValue *,       GVariant *,           gpointer);
static GVariant *rgba_to_s (const GValue *, const GVariantType *, gpointer);

static void folders_selector_get_value (GSettings *s, gchar *key, GtkWidget *w);

void img_selector_set_value (GtkWidget *, GSettings *);
void img_selector_get_value (GSettings *, gchar *, GtkWidget *);

void
add_key	(Plugin *p, GtkBox *box)
{
	GtkWidget *tmpBox, *ctl;

	BOX()

	switch (p->k.type)
	{
		case 'b':
			ctl = gtk_switch_new ();
			g_settings_bind (p->settings, p->k.name, ctl, "active", G_SETTINGS_BIND_DEFAULT);
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;
			
		case 'n': case 'q':		/* int16 uint16 */
		case 'i': case 'u':		/* int   uint   */
		case 'x': case 't':		/* int64 uint64 */
		case 'y': case 'd':		/* byte  double */
			ctl = gtk_spin_button_new_with_range ( p->k.min, p->k.max, p->k.inc);
			
			if (p->k.type == 'd')
				gtk_spin_button_set_digits (GTK_SPIN_BUTTON(ctl), p->k.digits);
				
			g_settings_bind (p->settings, p->k.name, ctl, "value", G_SETTINGS_BIND_DEFAULT);
			
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;

		case 'c':	/* Color */
			ctl = gtk_color_button_new ();
#if GTK_CHECK_VERSION(3,4,0)
			gtk_color_chooser_set_use_alpha (GTK_COLOR_CHOOSER(ctl), TRUE);
#else
			gtk_color_button_set_use_alpha (GTK_COLOR_BUTTON(ctl), TRUE);
#endif
			g_settings_bind_with_mapping (p->settings, p->k.name, ctl,
				"rgba", G_SETTINGS_BIND_DEFAULT, s_to_rgba, rgba_to_s, NULL, NULL);

			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;
			
		case 'p':	/* Picture */
		{
			GtkFileFilter *filter = gtk_file_filter_new ();
			gchar *signame = g_strdup_printf("changed::%s", p->k.name);

			ctl = gtk_file_chooser_button_new (_("Select a picture"), GTK_FILE_CHOOSER_ACTION_OPEN);
			g_object_set_data (G_OBJECT (ctl), "key", g_strdup(p->k.name));
			g_signal_connect ( ctl, "file-set", G_CALLBACK(img_selector_set_value), p->settings);
			g_signal_connect (p->settings, signame, G_CALLBACK(img_selector_get_value), ctl);
			g_free (signame);
			
			gtk_file_filter_add_pixbuf_formats (filter);
			gtk_file_chooser_set_filter ( GTK_FILE_CHOOSER (ctl), filter);
			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (ctl), g_settings_get_string (p->settings, p->k.name));
			
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;
		}
		case 'F':	/* Folder */
		{
			gchar *signame = g_strdup_printf("changed::%s", p->k.name);

			ctl = gtk_file_chooser_button_new (_("Select a directory"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
			g_object_set_data (G_OBJECT (ctl), "key", g_strdup(p->k.name));
			g_signal_connect ( ctl, "file-set", G_CALLBACK(img_selector_set_value), p->settings);
			g_signal_connect (p->settings, signame, G_CALLBACK(img_selector_get_value), ctl);
			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (ctl), g_settings_get_string (p->settings, p->k.name));
			g_free (signame);

			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;
		}
		case 'D':	/* Directories */
		{
			gchar *signame = g_strdup_printf("changed::%s", p->k.name);

			GtkWidget *box	   	= gtk_box_new (GTK_ORIENTATION_VERTICAL, 0),
					  *btnbox	= gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
			
			ctl = gtk_tree_view_new ();			
			gtk_tree_view_append_column (GTK_TREE_VIEW(ctl),
										 gtk_tree_view_column_new_with_attributes (_("Folders"),
																				   gtk_cell_renderer_text_new (),
																				   "text", 0, NULL));
			g_signal_connect (p->settings, signame, G_CALLBACK(folders_selector_get_value), ctl);
			gtk_box_pack_start (GTK_BOX(box), ctl, FALSE, FALSE, 0);
			g_free (signame);
			
			folders_selector_get_value (p->settings, p->k.name, ctl);
			
			/* Button ADD */
			ctl = gtk_button_new ();
			gtk_button_set_image (GTK_BUTTON(ctl),
								  gtk_image_new_from_stock (GTK_STOCK_ADD,
															GTK_ICON_SIZE_MENU));
			gtk_box_pack_start (GTK_BOX(btnbox), ctl, FALSE, FALSE, 0);
			/* Button DEL */
			ctl = gtk_button_new ();
			gtk_button_set_image (GTK_BUTTON(ctl),
								  gtk_image_new_from_stock (GTK_STOCK_DELETE,
															GTK_ICON_SIZE_MENU));
			gtk_box_pack_end (GTK_BOX(btnbox), ctl, FALSE, FALSE, 0);
			
			gtk_box_pack_end (GTK_BOX(box), btnbox, FALSE, FALSE, 0);
			
			gtk_box_pack_end (GTK_BOX(tmpBox), box, FALSE, FALSE, 0);
			
			break;
		}
		case 'f':	/* Font */
			ctl = gtk_font_button_new ();
			g_settings_bind (p->settings, p->k.name, ctl, "font-name", G_SETTINGS_BIND_DEFAULT);
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;
			
		case 's':	/* String */
			ctl = gtk_entry_new ();
			g_settings_bind (p->settings, p->k.name, ctl, "text", G_SETTINGS_BIND_DEFAULT);
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, TRUE, TRUE, 0);
			break;
		
		default:
			g_warning ("livewallpaper-config : Unrecognized data-type %c for key %s", p->k.type, p->k.name);
			break;
	}
}

void
reset_key (GtkWidget *w, GSettings *s)
{
	/* Reset the key joined to the GtkButton w in GSettings */
	g_settings_reset (s, g_object_get_data(G_OBJECT(w), "key"));
}

static void
folders_selector_get_value (GSettings *s, gchar *key, GtkWidget *w)
{
	gint i;
	gchar **folders = g_settings_get_strv (s,key);
	GtkTreeStore *store = gtk_tree_store_new (1, G_TYPE_STRING);
	
	for (i=0; folders[i]; i++)
	{
		GtkTreeIter iter;
		gtk_tree_store_append (store, &iter, NULL);
		gtk_tree_store_set 	  (store, &iter, 0, folders[i], -1);
	}
	
	gtk_tree_view_set_model (GTK_TREE_VIEW(w), GTK_TREE_MODEL(store));
	g_object_unref (G_OBJECT(store));
}

void
cb_bind_get_value (GSettings *s, gchar *key, GtkWidget *w)
{
	/* Get the identifier of the activated choice of the key in GSettings
	 * and set the active choice to this identifier in the GtkComboBoxText */
	gtk_combo_box_set_active (GTK_COMBO_BOX(w),
							  g_settings_get_enum (s, key));
}

void
cb_bind_set_value (GtkWidget *w, GSettings *s)
{
	/* Set the current value of the GtkComboBoxText to GSettings
	 * by the identifier of the value */
	g_settings_set_enum (s, g_object_get_data(G_OBJECT(w), "key"),
						 gtk_combo_box_get_active (GTK_COMBO_BOX(w)));
}

/* Inspirated by gnome-terminal (profile-editor.c ) */
static gboolean
s_to_rgba (GValue *value, GVariant *variant, gpointer p)
{
	const gchar *s;
	GdkRGBA color;
	(void) p;

	g_variant_get (variant, "&s", &s);
	gdk_rgba_parse (&color, s);

	g_value_set_boxed (value, &color);
	return TRUE;
}

/* Inspirated by gnome-terminal (profile-editor.c ) */
static GVariant *
rgba_to_s (const GValue *v, const GVariantType *t, gpointer p)
{
	(void) t; (void) p;
	return g_variant_new_string (gdk_rgba_to_string (g_value_get_boxed (v)));
}

void
img_selector_set_value (GtkWidget *w, GSettings *s)
{	
	gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
	if(filename) g_settings_set_string (s, g_object_get_data (G_OBJECT (w), "key"), filename);
	g_free (filename);
}

void
img_selector_get_value (GSettings *s, gchar *key, GtkWidget *w)
{	
	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (w), g_settings_get_string (s, key));
}
