/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "add_tab.h"
#define LWP_CONFIG
#include "lw-gui-common.h"

static void  show_plugin_info(GtkWidget *, GKeyFile  *);

GtkLicense get_gtk_license_from_license_name (gchar *name)
{
	if (! strcmp(name, "GPL2.0"))   return GTK_LICENSE_GPL_2_0;
	if (! strcmp(name, "GPL3.0"))   return GTK_LICENSE_GPL_3_0;
	if (! strcmp(name, "LGPL2.1"))  return GTK_LICENSE_LGPL_2_1;
	if (! strcmp(name, "LGPL3.0"))  return GTK_LICENSE_LGPL_3_0;
	if (! strcmp(name, "BSD"))      return GTK_LICENSE_BSD;
	if (! strcmp(name, "MIT/X11"))  return GTK_LICENSE_MIT_X11;
	if (! strcmp(name, "Artistic")) return GTK_LICENSE_ARTISTIC;

	return GTK_LICENSE_UNKNOWN;
}

#define KEY_EXISTS(key) g_key_file_has_key 			 (gkf, "Plugin", key, NULL)
#define KEY_STRVAL(key) g_key_file_get_locale_string (gkf, "Plugin", key, NULL, NULL)
static void
show_plugin_info (GtkWidget *w, GKeyFile *gkf)
{
	if (KEY_EXISTS("Name"))
	{
		GtkAboutDialog *dialog = GTK_ABOUT_DIALOG (gtk_about_dialog_new ());
		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(gtk_widget_get_toplevel(w)));
		
		gtk_about_dialog_set_program_name   (dialog, KEY_STRVAL("Name"));
		gtk_about_dialog_set_comments       (dialog, KEY_STRVAL("Description"));
		gtk_about_dialog_set_logo_icon_name (dialog, g_object_get_data(G_OBJECT(w), "icon"));
		gtk_about_dialog_set_authors        (dialog, (const gchar **)g_key_file_get_locale_string_list (gkf, "Plugin", "Authors", NULL, NULL, NULL));
		gtk_about_dialog_set_copyright      (dialog, KEY_STRVAL("Copyright"));
		gtk_about_dialog_set_wrap_license   (dialog, TRUE);
		
		if (KEY_EXISTS("License-type")) gtk_about_dialog_set_license_type (dialog, get_gtk_license_from_license_name (KEY_STRVAL("License-type")));
		if (KEY_EXISTS("License"))      gtk_about_dialog_set_license (dialog, KEY_STRVAL("License"));
		if (KEY_EXISTS("Website"))      gtk_about_dialog_set_website (dialog, KEY_STRVAL("Website"));

		switch (gtk_dialog_run (GTK_DIALOG (dialog)))
		{
			case GTK_RESPONSE_CANCEL:
			case GTK_RESPONSE_DELETE_EVENT: 
				gtk_widget_destroy(GTK_WIDGET(dialog));
				break;
		};
	}
	else
	{
		const gchar *authors[] = {"Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>", "Aurélien RIVIERE <aurelien.riv@gmail.com>", NULL};

		gtk_show_about_dialog ( GTK_WINDOW(gtk_widget_get_toplevel(w)),
			"authors",  		authors,
			"comments", 		_("Replace your boring default wallpaper with a completely animated OpenGL wallpaper."),
			"copyright", 		"Copyright © 2012 Maximilian Schnarr",
			"license-type",		GTK_LICENSE_GPL_3_0,
			"logo_icon_name", 	"livewallpaper",
			"program-name", 	_("Livewallpaper"),
			"website", 			"http://launchpad.net/livewallpaper",
			NULL);
	}
}

void
add_tab (GtkNotebook *notebook, Plugin *p)
{
	static GSList *group = NULL;
	GSettings *settings  = g_settings_new (GSETTINGS_PREFIX);
	GtkWidget *box       = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5),
			  *tmpBox    = gtk_box_new (GTK_ORIENTATION_VERTICAL,   5),
			  *ctl 		 = gtk_button_new ();

	/* Add a button to display plugin's informations in a toplevel window */
	gtk_button_set_image (GTK_BUTTON(ctl),
						  gtk_image_new_from_stock (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU));
	g_signal_connect ( G_OBJECT (ctl), "clicked", G_CALLBACK(show_plugin_info), p->gkf);
	gtk_box_pack_end(GTK_BOX(box), ctl, FALSE, FALSE, 0);
	
	if (strcmp(p->id, "livewallpaper"))
	{
		gchar *plugin_icon = g_strconcat ("livewallpaper-", p->id, NULL);
			g_object_set_data (G_OBJECT (ctl), "icon", g_strdup(plugin_icon));
			gtk_box_pack_start(GTK_BOX(box), gtk_image_new_from_icon_name (plugin_icon, GTK_ICON_SIZE_DIALOG), FALSE, FALSE, 0);
		g_free (plugin_icon);
		
		gtk_box_pack_start (GTK_BOX(box), gtk_label_new (g_key_file_get_locale_string(p->gkf,"Plugin","Name",NULL,NULL)), TRUE, TRUE, 0);

		/* Create a GtkRadioButton and join it to the GSList group */
		ctl = gtk_radio_button_new (group);
		group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (ctl));
		
		gtk_widget_set_tooltip_text (ctl, _("Define which plugin is currently enabled."));
		
		/* Bind the plugin's name to the widget and connect to the signals */
		g_object_set_data (G_OBJECT (ctl), "plugin",  g_strdup(p->id));
		g_signal_connect  (G_OBJECT (ctl), "toggled",                G_CALLBACK(_active_plugin_set), settings);
		g_signal_connect  (settings,       "changed::active-plugin", G_CALLBACK(_active_plugin_get), group);
		_active_plugin_get (settings, NULL, group);

		gtk_notebook_append_page (notebook, GTK_WIDGET(p->nb), box);
	}
	else
	{
		gtk_box_pack_start(GTK_BOX(box), gtk_image_new_from_icon_name ("livewallpaper", GTK_ICON_SIZE_DIALOG), FALSE, FALSE, 0);
		gtk_box_pack_start (GTK_BOX(box), gtk_label_new ("Livewallpaper"), TRUE, TRUE, 0);
		
		ctl = gtk_switch_new ();
		g_settings_bind (settings, "active", ctl, "active", G_SETTINGS_BIND_DEFAULT);
		g_signal_connect (G_OBJECT(ctl), "activate", G_CALLBACK (livewallpaper_button_activated), settings);
		gtk_widget_set_tooltip_text (ctl, _("Show or hide the live wallpaper. This will start the livewallpaper application if it is not already running."));
		
		g_bus_watch_name (G_BUS_TYPE_SESSION, "net.launchpad.Livewallpaper", G_BUS_NAME_WATCHER_FLAGS_NONE,
					      (GBusNameAppearedCallback) core_appeared, (GBusNameVanishedCallback) core_vanished,
					      ctl, NULL);

		/* Insert this page at first position */
		gtk_notebook_insert_page (notebook, GTK_WIDGET(p->nb), box, 0);
	}
	gtk_box_pack_end (GTK_BOX(tmpBox), ctl, TRUE, FALSE, 5);
	gtk_box_pack_end (GTK_BOX(box), tmpBox, FALSE, FALSE, 10);
	gtk_widget_show_all(box);
}
