/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "loader.h"

#define SPACE 	 	  "%*[ \t]"
#define STR_ATTR(len) "%*c" "%"#len"[^\">]" "%*c"
#define STR_TEXT(len) "%*c" "%"#len"[^\"</\"]"

static guint nb_decimals  (gdouble x);
static void  load_enum    (Plugin *, GtkBox *);

static guint
nb_decimals  (gdouble x)
{
	guint n = 7, /* number of decimals */
		  b = 10;
	x = x*10e6;

	while (! ((guint)x % b) && n--)
		b *= 10;

	return n;
}

static void
load_enum (Plugin *p, GtkBox *box)
{
	gchar *filename = g_strdup(p->schemafile);
	FILE *file = NULL;
	
	guint step = 1;			/* 1 = parse gschema; 2 = parse enum (if any) */
	gboolean pass = FALSE;  /* !pass ? look for the enum and ignore found values : enum found, read its values */

	GtkWidget *tmpBox, *ctl;	
			
	read_file:
		if(! (file = fopen(filename, "r")))
			perror(filename);
		else
		{
			gchar str[1024];
	
			/* Read the entire file line by line */
			while (fgets(str, 1024, file))
			{
				if (strstr(str, "<enum"))
				{
					gchar enum_id[128];
					sscanf(strpbrk(str,"<"), "<enum" SPACE "id=" STR_ATTR(127), enum_id);

					if((pass = !strcmp(enum_id, p->k.enum_id)))
					{
						BOX();
						ctl = gtk_combo_box_text_new ();
						/* Join the key name to the GtkComboBoxText */
						g_object_set_data (G_OBJECT(ctl), "key", g_strdup(p->k.name));
					}
				}
				else if (pass && strstr(str, "<value"))
				{
					gint value; gchar nick[64];

					if ( sscanf(strpbrk(str,"n"), "nick=" STR_ATTR(63) SPACE "value=%*c%i%*c", nick, &value)  != 2)
						 sscanf(strpbrk(str,"v"), "value=%*c%i%*c" SPACE "nick=" STR_ATTR(63), &value, nick);
						 
					/* Add the found values to the GtkComboBoxText with value as identifier and nick as displayed string (translated) */
					gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT(ctl), value, _(nick));
				}
				else if (pass && strstr(str,"</enum>"))
				{		
					/* Bind the "changed "signal to the widget and the "changed::name" to gsettings */
					gchar *signame = g_strdup_printf("changed::%s", p->k.name);
					g_signal_connect (ctl, "changed", G_CALLBACK(cb_bind_set_value), p->settings);
					g_signal_connect (p->settings, signame, G_CALLBACK(cb_bind_get_value), ctl);
					g_free (signame);
					
					/* Update the widget */
					cb_bind_get_value (p->settings, p->k.name, ctl);

					gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
					
					break;
				}
				else if (strstr(str, "<lwp:tab"))
					break;
			}

			g_free(filename);
			fclose(file);
			file = NULL;
		}
		
		if (!pass && step == 1)
		{
			step = 2;
			filename = g_strdup(p->enumfile);
			goto read_file;
		}
}

void
parse_schema (GtkNotebook *notebook, Plugin *p)
{
	GtkWidget *tab = NULL;	/* The tab which will countain the keys. Reloaded for each tab */
	gchar str[1024];
	
	FILE *file = fopen(p->schemafile, "r");
	if(!file)
	{
		perror(p->schemafile);
		return;
	}
	
	/* The per-plugin notebook */
	p->nb  = GTK_NOTEBOOK(gtk_notebook_new());
	gtk_notebook_set_scrollable (p->nb, TRUE);

	while (fgets(str, 1024, file))
	{
		if (strstr(str, "<lwp:tab"))
		{
			/* Create a scolled area for the keys and add it in a new tab */
			GtkWidget *scroll = gtk_scrolled_window_new (NULL, NULL);

			gchar tab_name[32];
			sscanf(strpbrk(str,"n"), "name=" STR_ATTR(31), tab_name);

			tab = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW (scroll), tab);
			gtk_notebook_append_page (p->nb, scroll, gtk_label_new(_(tab_name)));
		}

		else if (strstr(str, "<lwp:separator"))
			/* Add an horizontal separator to separate widgets inside a same tab */
			gtk_box_pack_start (GTK_BOX(tab),
							  gtk_separator_new(GTK_ORIENTATION_HORIZONTAL),
							  FALSE, FALSE, 2);
		
		else if (strstr(str,"<key "))
		{
			if (strstr(str,"type="))
			{
				/* Get the type and the name of the new key,
				 * and set the number of digits and the incrementation
				 * to a default value */
				if ( sscanf(strpbrk(str,"t"), "type=%*c%c%*c" SPACE "name=" STR_ATTR(31), &p->k.type, p->k.name) != 2)
					 sscanf(strpbrk(str,"n"), "name=" STR_ATTR(31) SPACE "type=%*c%c%*c", p->k.name, &p->k.type);
				p->k.digits = 6;
				p->k.inc    = (p->k.type == 'd') ? 10e-7 : 1;
			}
			else if (strstr(str, "enum="))
			{
				/* Get the name of an enum key and the associated enum,
				 * and set type to 'e' (not reserved by glib types and
				 * permit to know the key needs load_enum instead of add_key) */
				if ( sscanf(strpbrk(str,"n"), "name=" STR_ATTR(31)  SPACE "enum=" STR_ATTR(127), p->k.name, p->k.enum_id) != 2)
					 sscanf(strpbrk(str,"e"), "enum=" STR_ATTR(127) SPACE "name=" STR_ATTR(31),  p->k.enum_id, p->k.name);
				p->k.type = 'e';
			}
			
			p->k.skip = FALSE;
		}
		else if (strstr(str,"<lwp:type"))
		{
			gchar buffer[16];
			sscanf(strpbrk(str,">"), STR_TEXT(15), buffer);

			if      (! strcmp(buffer, "image"))   p->k.type = 'p';
			else if (! strcmp(buffer, "color"))   p->k.type = 'c';
			else if (! strcmp(buffer, "font"))    p->k.type = 'f';
			else if (! strcmp(buffer, "folder"))  p->k.type = 'F';
			else if (! strcmp(buffer, "folders")) p->k.type = 'D';
		}
		
		else if (strstr(str,"<lwp:skip"))
			p->k.skip = TRUE;
			
		else if (strstr(str,"<range"))
			sscanf(strpbrk(str,"m"), "min=%*c%f%*c" SPACE "max=%*c%f%*c", &p->k.min, &p->k.max);

		else if (strstr(str,"<summary>"))
			sscanf(strpbrk(str,">"), STR_TEXT(255), p->k.summary);

		else if (strstr(str,"<description>"))
			sscanf(strpbrk(str,">"), STR_TEXT(511), p->k.description);

		else if (strstr(str, "<lwp:increment"))
			/* The increment step of the buttons + and - when we do a button1 click 
			 * button2 click = 10*button1 click */
			sscanf(strpbrk(str,">"), ">%f", &p->k.inc);

		else if (strstr(str, "<lwp:digits"))
			/* The number of decimals displayed */
			sscanf(strpbrk(str,">"), ">%u", &p->k.digits);

		else if (strstr(str,"</key>") && !p->k.skip)
		{
			if (p->k.type == 'e')
				load_enum (p, GTK_BOX(tab));
			else
			{
				if (p->k.type == 'd')
					if ( nb_decimals (p->k.inc) > p->k.digits)
						p->k.inc = pow (10.0, - (gdouble)p->k.digits);
				add_key (p, GTK_BOX(tab));
			}
		}
	}

	fclose (file);
	
	/* Add the per-plugin notebook to the main notebook and
	 * add the GtkRadioButton which permit to select the active-plugin */
	add_tab (notebook, p);
}
