/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "plugins.h"

static const gchar*
find_plugin_datadir (const gchar *plugin)
{
	if (!strcmp(plugin, "livewallpaper"))
		return BASEDIR;
	else
	{
		gchar *dir = g_build_filename (g_get_home_dir(), ".livewallpaper", "plugins", plugin, NULL);
		if( g_file_test (dir, G_FILE_TEST_IS_DIR))
		{
			g_free (dir);
			return g_get_home_dir();
		}

		g_free (dir);
		dir = g_build_filename (G_DIR_SEPARATOR_S "usr", "local", "lib", "livewallpaper", "plugins", plugin, NULL);
		if( g_file_test (dir, G_FILE_TEST_IS_DIR))
		{
			g_free (dir);
			return G_DIR_SEPARATOR_S"usr"G_DIR_SEPARATOR_S"local"G_DIR_SEPARATOR_S;
		}
		
		g_free (dir);
		dir = g_build_filename (G_DIR_SEPARATOR_S "usr", "lib", "livewallpaper", "plugins", plugin, NULL);
		if( g_file_test (dir, G_FILE_TEST_IS_DIR))
		{
			g_free (dir);
			return G_DIR_SEPARATOR_S"usr"G_DIR_SEPARATOR_S;
		}
	}

	return NULL;	
}

void
init_plugins (GtkNotebook *w)
{
	guint i;
	const gchar **schemas = (const gchar **) g_settings_list_schemas();

	/* For each installed schema,
	 * if the schema has the prefix of a livewallpaper's plugin,
	 * load it */
	for (i=0; schemas[i]; i++)
		if (g_str_has_prefix (schemas[i], GSETTINGS_PREFIX))
		{
			Plugin p;
			gchar *filename, *filename2;
			
			p.id = (strcmp(schemas[i], GSETTINGS_PREFIX)) ? schemas[i]+GSETTINGS_PREFIX_LEN : "livewallpaper";
			if(! (p.path = find_plugin_datadir (p.id)))
			{
				g_critical("Could not find data dir of the %s plugin", p.id);
				continue;
			}

			p.settings = g_settings_new (schemas[i]);
			
			filename2 = g_strconcat (p.id, ".plugin", NULL);
			if (! strcmp (p.path, g_get_home_dir ()))
			{
				filename  = g_build_filename(p.path, ".livewallpaper", "plugins", p.id, filename2, NULL);
				g_free (filename2);
				filename2 = g_strconcat(p.path,G_DIR_SEPARATOR_S".local",NULL);
			}
			else
			{
				filename  = g_build_filename(p.path, "lib", "livewallpaper", "plugins", p.id, filename2, NULL);
				g_free (filename2);
				filename2 = g_strdup(p.path);
			}
			p.gkf = g_key_file_new ();
			g_key_file_load_from_file (p.gkf, filename, G_KEY_FILE_NONE, NULL);
			g_free (filename);

			p.schemafile = g_strconcat (filename2, G_DIR_SEPARATOR_S"share"G_DIR_SEPARATOR_S"glib-2.0"G_DIR_SEPARATOR_S"schemas"G_DIR_SEPARATOR_S, schemas[i], ".gschema.xml", NULL);
			p.enumfile   = g_strconcat (filename2, G_DIR_SEPARATOR_S"share"G_DIR_SEPARATOR_S"glib-2.0"G_DIR_SEPARATOR_S"schemas"G_DIR_SEPARATOR_S, schemas[i], ".enums.xml", NULL);
			g_free (filename2);
			
			parse_schema (w, &p);		
		}
}
