/*
 *
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>
 *
 */

/**
 * SECTION: program
 * @Short_description: program
 *
 * TODO
 */

#include <livewallpaper/core.h>
#include <string.h>


#define LW_PROGRAM_GET_PRIVATE(obj)  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), LW_TYPE_PROGRAM, LwProgramPrivate))

struct _LwProgramPrivate
{
	guint name;
};

G_DEFINE_TYPE(LwProgram, lw_program, G_TYPE_OBJECT)

/**
 * lw_program_get_name:
 * @self: A #LwProgram
 *
 * Returns: TODO
 */
guint
lw_program_get_name(LwProgram *self)
{
	return self->priv->name;
}

/**
 * lw_program_attach_shader:
 * @self: A #LwProgram
 * @shader: TODO
 *
 * TODO
 */
void
lw_program_attach_shader(LwProgram *self, LwShader *shader)
{
	glAttachShader(self->priv->name, lw_shader_get_name(shader));
}

/**
 * lw_program_link:
 * @self: A #LwProgram
 *
 * TODO
 *
 * Returns: TODO
 */
gboolean
lw_program_link(LwProgram *self)
{
	int status;

	glLinkProgram(self->priv->name);

	/* Handle errors */
	glGetProgramiv(self->priv->name, GL_LINK_STATUS, &status);
	if(status == GL_FALSE)
	{
		int log_length;
		gchar *log_buffer;

		glGetProgramiv(self->priv->name, GL_INFO_LOG_LENGTH, &log_length);
		log_buffer = g_malloc(log_length * sizeof(gchar));

		glGetProgramInfoLog(self->priv->name, log_length, NULL, log_buffer);

		g_warning("%s", log_buffer);

		g_free(log_buffer);
		return FALSE;
	}

	return TRUE;
}

/**
 * lw_program_get_attrib_location:
 * @self: A #LwProgram
 * @name: TODO
 *
 * TODO
 *
 * Returns: TODO
 */
gint
lw_program_get_attrib_location(LwProgram *self, const gchar *name)
{
	return glGetAttribLocation(self->priv->name, name);
}

/**
 * lw_program_get_uniform_location:
 * @self: A #LwProgram
 * @name: TODO
 *
 * TODO
 *
 * Returns: TODO
 */
gint
lw_program_get_uniform_location(LwProgram *self, const gchar *name)
{
	return glGetUniformLocation(self->priv->name, name);
}

/**
 * lw_program_enable:
 * @self: A #LwProgram
 *
 * TODO
 */
void
lw_program_enable(LwProgram *self)
{
	glUseProgram(self->priv->name);
}

/**
 * lw_program_disable:
 * @self: A #LwProgram
 *
 * TODO
 */
void
lw_program_disable(LwProgram *self)
{
	(void) self;

	glUseProgram(0);
}

static void
lw_program_init(LwProgram *self)
{
	self->priv = LW_PROGRAM_GET_PRIVATE(self);

	self->priv->name = glCreateProgram();
}

static void
lw_program_finalize(GObject *object)
{
	LwProgram *self = LW_PROGRAM(object);

	if(self->priv->name)
		glDeleteProgram(self->priv->name);

	/* Chain up to the parent class */
	G_OBJECT_CLASS(lw_program_parent_class)->finalize(object);
}

static void
lw_program_class_init(LwProgramClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

	gobject_class->finalize = lw_program_finalize;

	g_type_class_add_private(klass, sizeof(LwProgramPrivate));
}

