/*
 *
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>
 *
 */

/**
 * SECTION: shader
 * @Short_description: shader
 *
 * TODO
 */

#include <livewallpaper/core.h>
#include <string.h>


#define LW_SHADER_GET_PRIVATE(obj)  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), LW_TYPE_SHADER, LwShaderPrivate))

struct _LwShaderPrivate
{
	guint name;
	guint type;
};

G_DEFINE_TYPE(LwShader, lw_shader, G_TYPE_OBJECT)

/**
 * lw_shader_new_from_file:
 * @filename: TODO
 * @type: TODO
 * @error: TODO
 *
 * TODO
 *
 * Returns: TODO
 */
LwShader*
lw_shader_new_from_file(const gchar *filename, guint type, GError **error)
{
	LwShader *shader;
	gchar *source;

	if(!g_file_get_contents(filename, &source, NULL, error)) return NULL;

	shader = lw_shader_new_from_string(source, type);
	g_free(source);

	return shader;
}

/**
 * lw_shader_new_from_string:
 * @source: TODO
 * @type: TODO
 *
 * TODO
 *
 * Returns: TODO
 */
LwShader*
lw_shader_new_from_string(const gchar *source, guint type)
{
	LwShader *shader = g_object_new(LW_TYPE_SHADER, NULL);
	int source_length = strlen(source);

	shader->priv->name = glCreateShader(type);
	shader->priv->type = type;

	glShaderSource(shader->priv->name, 1, &source, &source_length);

	return shader;
}

/**
 * lw_shader_get_name:
 * @self: A #LwShader
 *
 * Returns: TODO
 */
guint
lw_shader_get_name(LwShader *self)
{
	return self->priv->name;
}

/**
 * lw_shader_get_shader_type:
 * @self: A #LwShader
 *
 * Returns: TODO
 */
guint
lw_shader_get_shader_type(LwShader *self)
{
	return self->priv->type;
}

/**
 * lw_shader_compile:
 * @self: A #LwShader
 *
 * TODO
 *
 * Returns: TODO
 */
gboolean
lw_shader_compile(LwShader *self)
{
	int status;

	glCompileShader(self->priv->name);

	/* Handle errors */
	glGetShaderiv(self->priv->name, GL_COMPILE_STATUS, &status);
	if(status == GL_FALSE)
	{
		int log_length;
		gchar *log_buffer;

		glGetShaderiv(self->priv->name, GL_INFO_LOG_LENGTH, &log_length);
		log_buffer = g_malloc(log_length * sizeof(gchar));

		glGetShaderInfoLog(self->priv->name, log_length, NULL, log_buffer);

		g_warning("%s", log_buffer);

		g_free(log_buffer);
		return FALSE;
	}

	return TRUE;
}

static void
lw_shader_init(LwShader *self)
{
	self->priv = LW_SHADER_GET_PRIVATE(self);

	self->priv->name = 0;
}

static void
lw_shader_finalize(GObject *object)
{
	LwShader *self = LW_SHADER(object);

	if(self->priv->name)
		glDeleteShader(self->priv->name);

	/* Chain up to the parent class */
	G_OBJECT_CLASS(lw_shader_parent_class)->finalize(object);
}

static void
lw_shader_class_init(LwShaderClass *klass)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);

	gobject_class->finalize = lw_shader_finalize;

	g_type_class_add_private(klass, sizeof(LwShaderPrivate));
}

