/*
 *
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>
 *
 */

#include "config.h"

#include <glib.h>
#include <glib/gi18n.h>
#include <gio/gio.h>

#include "command-line.h"


#define LW_COMMAND_LINE_GET_PRIVATE(obj)  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), LW_TYPE_COMMAND_LINE, LwCommandLinePrivate))

struct _LwCommandLinePrivate
{
	gboolean show_fps;
	gboolean lower;
	gboolean if_autostart;
};

G_DEFINE_TYPE(LwCommandLine, lw_command_line, G_TYPE_OBJECT)

static GOptionEntry options[] = {
	/* Show FPS */
	{
		"show-fps", 'f', 0, G_OPTION_ARG_NONE, NULL,
		N_("Show FPS in the lower right corner"), NULL
	},

	/* Lower */
	{
		"lower", 0, 0, G_OPTION_ARG_NONE, NULL,
		N_("Move the window behind all other desktop windows"), NULL
	},

	/* If Autostart */
	{
		"if-autostart", 0, 0, G_OPTION_ARG_NONE, NULL,
		N_("Start the livewallpaper if autostart is enabled"), NULL
	},

	{ NULL }
};

gboolean
lw_command_line_parse(LwCommandLine *self, gint *argc, gchar ***argv)
{
	GError *error = NULL;
	GOptionContext *context;

	/* Set arg_data of all options */
	options[0].arg_data = &self->priv->show_fps;
	options[1].arg_data = &self->priv->lower;
	options[2].arg_data = &self->priv->if_autostart;

	context = g_option_context_new(NULL);
	g_option_context_add_main_entries(context, options, GETTEXT_PACKAGE);

	if(!g_option_context_parse(context, argc, argv, &error))
	{
		g_critical("Could not parse command line: %s\n", error->message);
		g_option_context_free(context);
		g_error_free(error);
		return FALSE;
	}

	g_option_context_free(context);
	return TRUE;
}

gboolean
lw_command_line_get_show_fps(LwCommandLine *self)
{
	return self->priv->show_fps;
}

gboolean
lw_command_line_get_lower(LwCommandLine *self)
{
	return self->priv->lower;
}

gboolean
lw_command_line_get_if_autostart(LwCommandLine *self)
{
	return self->priv->if_autostart;
}

static void
lw_command_line_init(LwCommandLine *self)
{
	self->priv = LW_COMMAND_LINE_GET_PRIVATE(self);

	self->priv->show_fps = FALSE;
	self->priv->lower = FALSE;
	self->priv->if_autostart = FALSE;
}

static void
lw_command_line_class_init(LwCommandLineClass *klass)
{
	g_type_class_add_private(klass, sizeof(LwCommandLinePrivate));
}

