/*
 *
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>
 *
 */

#include "config.h"

#include <glib.h>
#include <gdk/gdk.h>
#include <girepository.h>

#include "command-line.h"
#include "application.h"


gint
main(gint argc, gchar *argv[])
{
	GApplication *g_app;
	LwCommandLine *command_line;
	LwApplication *app;
	gchar *filename;
	GError *error = NULL;
	gint retval;

	/* Initialize external libraries */
	g_type_init();
	gdk_init(&argc, &argv);

	/* Parse command line */
	command_line = g_object_new(LW_TYPE_COMMAND_LINE, NULL);
	if(!lw_command_line_parse(command_line, &argc, &argv)) return 1;

	/* Check for application uniqueness */
	g_app = g_application_new("net.launchpad.Livewallpaper", G_APPLICATION_IS_SERVICE);
	if(!g_application_register(g_app, NULL, &error))
	{
		g_critical("Could not register application: %s", error->message);
		g_error_free(error);
		return 1;
	}

	/* Enable local settings */
	filename = g_build_filename(g_get_home_dir(), ".local", "share", "glib-2.0", "schemas", NULL);
	g_setenv("GSETTINGS_SCHEMA_DIR", filename, TRUE);
	g_free(filename);

	/* Load required repositories */
	if(!g_irepository_require(g_irepository_get_default(), "Peas", "1.0", 0, &error))
	{
		g_critical("Could not load Peas repository: %s", error->message);
		g_error_free(error);
		return 1;
	}

	filename = g_build_filename(LIBDIR, "livewallpaper", "girepository-1.0", NULL);
	if(!g_irepository_require_private(g_irepository_get_default(), filename, "LW", "1.0", 0, &error))
	{
		g_critical("Could not load LW repository: %s", error->message);
		g_error_free(error);
		return 1;
	}
	g_free(filename);

	/* Run livewallpaper application */
	app = g_object_new(LW_TYPE_APPLICATION, NULL);

	retval = lw_application_run(app, command_line);

	/* Exit application */
	g_object_unref(app);
	g_object_unref(command_line);
	g_object_unref(g_app);

	return retval;
}

